% counterfact: Generates counterfactual data from a three-variable VAR with
% time-varying parameters and stochastic volatility under the following
% scenarios:
%
%      (1) Without monetary policy shocks from period Tb forward.
%      (2) With the volatility of mp shocks fixed from Tb forward.
%      (3) With the mp rule fixed from Tb forward.
%      (4) With the mp rule and volatility of mp shocks fixed from Tb
%      forward.
%      (5) With no mp shocks for the entire sample.
%      (6) With the 1990:1 mp rule in place for the entire sample.
%      (7) With the 1990:1 mp rule in place from Tb forward.

% =========================================================================
% Load results from posterior simulation and choose breakpoint Tb
% =========================================================================

load betaT
load tvpvarout

Tb = 121;
% Tb = time index of hypothesized break:
%        21=1975, 41=1980, 61=1985, 81=1990, 101=1995, 121=2000, 141=2005

% =========================================================================
% Set up output matrices
% =========================================================================

pdata0 = zeros(nsweeps,T);
gdata0 = zeros(nsweeps,T);
rdata0 = zeros(nsweeps,T);

pdata1 = zeros(nsweeps,T);
gdata1 = zeros(nsweeps,T);
rdata1 = zeros(nsweeps,T);

pdata2 = zeros(nsweeps,T);
gdata2 = zeros(nsweeps,T);
rdata2 = zeros(nsweeps,T);

pdata3 = zeros(nsweeps,T);
gdata3 = zeros(nsweeps,T);
rdata3 = zeros(nsweeps,T);

pdata4 = zeros(nsweeps,T);
gdata4 = zeros(nsweeps,T);
rdata4 = zeros(nsweeps,T);

pdata5 = zeros(nsweeps,T);
gdata5 = zeros(nsweeps,T);
rdata5 = zeros(nsweeps,T);

pdata6 = zeros(nsweeps,T);
gdata6 = zeros(nsweeps,T);
rdata6 = zeros(nsweeps,T);

pdata7 = zeros(nsweeps,T);
gdata7 = zeros(nsweeps,T);
rdata7 = zeros(nsweeps,T);

% =========================================================================
% Re-generate actual history as a check
% =========================================================================

for sweep = 1:nsweeps
    
    BTmat = betaT(:,:,sweep);
    
    xt = xdata(1,:);
    
    for t = 1:T
        
        Xpt = kron(eye(3),xt);
        
        Bt = BTmat(:,t);
        
        Ct = [ cppT(sweep,t) -cpgT(sweep,t) -cprT(sweep,t) ; ...
               -cgpT(sweep,t) cggT(sweep,t) -cgrT(sweep,t) ; ...
               -crpT(sweep,t) -crgT(sweep,t) crrT(sweep,t) ];
           
        Dt = diag([deltapT(sweep,t) deltagT(sweep,t) deltarT(sweep,t) ]);
        
        xit = [ xipT(sweep,t) ; xigT(sweep,t) ; xirT(sweep,t) ];
        
        ut = Ct\(Dt*xit);
        
        yt = Xpt*Bt + ut;
        
        pdata0(sweep,t) = yt(1);
        gdata0(sweep,t) = yt(2);
        rdata0(sweep,t) = yt(3);
        
        xt(:,5:4+3*(q-1)) = xt(:,2:1+3*(q-1));
        xt(:,2:4) = yt';
    
    end
    
end

% =========================================================================
% Counterfactual with no monetary policy shocks from Tb forward
% =========================================================================

for sweep = 1:nsweeps
    
    BTmat = betaT(:,:,sweep);
    
    xt = xdata(1,:);
    
    for t = 1:T
        
        Xpt = kron(eye(3),xt);
        
        Bt = BTmat(:,t);
        
        Ct = [ cppT(sweep,T) -cpgT(sweep,t) -cprT(sweep,t) ; ...
               -cgpT(sweep,t) cggT(sweep,T) -cgrT(sweep,t) ; ...
               -crpT(sweep,t) -crgT(sweep,t) crrT(sweep,t) ];
           
        Dt = diag([deltapT(sweep,t) deltagT(sweep,t) deltarT(sweep,t) ]);
        
        xit = [ xipT(sweep,t) ; xigT(sweep,t) ; xirT(sweep,t) ];
        
        if t > Tb-q-1
            xit(3) = 0;
        end
        
        ut = Ct\(Dt*xit);
        
        yt = Xpt*Bt + ut;
        
        pdata1(sweep,t) = yt(1);
        gdata1(sweep,t) = yt(2);
        rdata1(sweep,t) = yt(3);
        
        xt(:,5:4+3*(q-1)) = xt(:,2:1+3*(q-1));
        xt(:,2:4) = yt';
    
    end
    
end

% =========================================================================
% Counterfactual with no change in volatility of monetary policy shocks
% from Tb forward
% =========================================================================

for sweep = 1:nsweeps
    
    BTmat = betaT(:,:,sweep);
    
    xt = xdata(1,:);
    
    for t = 1:T
        
        Xpt = kron(eye(3),xt);
        
        Bt = BTmat(:,t);
        
        Ct = [ cppT(sweep,t) -cpgT(sweep,t) -cprT(sweep,t) ; ...
               -cgpT(sweep,t) cggT(sweep,t) -cgrT(sweep,t) ; ...
               -crpT(sweep,t) -crgT(sweep,t) crrT(sweep,t) ];
           
        Dt = diag([deltapT(sweep,t) deltagT(sweep,t) deltarT(sweep,t) ]);
        
        if t > Tb-q-1
            Dt(3,3) = deltarT(sweep,Tb-q);
        end
        
        xit = [ xipT(sweep,t) ; xigT(sweep,t) ; xirT(sweep,t) ];
        
        ut = Ct\(Dt*xit);
        
        yt = Xpt*Bt + ut;
        
        pdata2(sweep,t) = yt(1);
        gdata2(sweep,t) = yt(2);
        rdata2(sweep,t) = yt(3);
        
        xt(:,5:4+3*(q-1)) = xt(:,2:1+3*(q-1));
        xt(:,2:4) = yt';
    
    end
    
end

% =========================================================================
% Counterfactual with no change in monetary policy rule from Tb forward
% =========================================================================

for sweep = 1:nsweeps
    
    BTmat = betaT(:,:,sweep);
    
    Bt = BTmat(:,Tb-q);
    
    Ct = [ cppT(sweep,Tb-q) -cpgT(sweep,Tb-q) -cprT(sweep,Tb-q) ; ...
           -cgpT(sweep,Tb-q) cggT(sweep,Tb-q) -cgrT(sweep,Tb-q) ; ...
           -crpT(sweep,Tb-q) -crgT(sweep,Tb-q) crrT(sweep,Tb-q) ];
    
    Gamma3mat = zeros(1,1+3*q);
                    
    Bmatt = reshape(Bt,1+3*q,3);
            
    Bmatt = Bmatt';
            
    gammavec = Ct*Bmatt(:,1);
            
    Gamma3mat(1,1) = gammavec(3);
            
    for p = 1:q
                
        Gammamat = Ct*Bmatt(:,2+3*(p-1):4+3*(p-1));
                
        Gamma3mat(1,2+3*(p-1):4+3*(p-1)) = Gammamat(3,:);
                
    end
    
    xt = xdata(1,:);
    
    for t = 1:T
        
        Xpt = kron(eye(3),xt);
        
        Bt = BTmat(:,t);
        
        Ct = [ cppT(sweep,t) -cpgT(sweep,t) -cprT(sweep,t) ; ...
               -cgpT(sweep,t) cggT(sweep,t) -cgrT(sweep,t) ; ...
               -crpT(sweep,t) -crgT(sweep,t) crrT(sweep,t) ];
           
        Dt = diag([deltapT(sweep,t) deltagT(sweep,t) deltarT(sweep,t) ]);
        
        xit = [ xipT(sweep,t) ; xigT(sweep,t) ; xirT(sweep,t) ];
        
        if t > Tb-q-1
            
            Btildamatt = zeros(3,1+3*q);
            
            Ctildat = [ cppT(sweep,t) -cpgT(sweep,t) -cprT(sweep,t) ; ...
                        -cgpT(sweep,t) cggT(sweep,t) -cgrT(sweep,t) ; ...
                        -crpT(sweep,Tb-q) -crgT(sweep,Tb-q) crrT(sweep,Tb-q) ];
                    
            Bmatt = reshape(Bt,1+3*q,3);
            
            Bmatt = Bmatt';
            
            gammavec = Ct*Bmatt(:,1);
            gammavec(3) = Gamma3mat(1,1);
            muvec = Ctildat\gammavec;
            
            Btildamatt(:,1) = muvec;
            
            for p = 1:q
                
                Gammamat = Ct*Bmatt(:,2+3*(p-1):4+3*(p-1));
                Gammamat(3,:) = Gamma3mat(1,2+3*(p-1):4+3*(p-1));
                Btildamatt(:,2+3*(p-1):4+3*(p-1)) = Ctildat\Gammamat;
                
            end
            
            Bt = reshape(Btildamatt',3+9*q,1);
            Ct = Ctildat;
            
        end
        
        ut = Ct\(Dt*xit);
        
        yt = Xpt*Bt + ut;
        
        pdata3(sweep,t) = yt(1);
        gdata3(sweep,t) = yt(2);
        rdata3(sweep,t) = yt(3);
        
        xt(:,5:4+3*(q-1)) = xt(:,2:1+3*(q-1));
        xt(:,2:4) = yt';
    
    end
    
end

% =========================================================================
% Counterfactual with no change in monetary policy rule and volatility of
% monetary policy shocks from Tb forward
% =========================================================================

for sweep = 1:nsweeps
    
    BTmat = betaT(:,:,sweep);
    
    Bt = BTmat(:,Tb-q);
    
    Ct = [ cppT(sweep,Tb-q) -cpgT(sweep,Tb-q) -cprT(sweep,Tb-q) ; ...
           -cgpT(sweep,Tb-q) cggT(sweep,Tb-q) -cgrT(sweep,Tb-q) ; ...
           -crpT(sweep,Tb-q) -crgT(sweep,Tb-q) crrT(sweep,Tb-q) ];
    
    Gamma3mat = zeros(1,1+3*q);
                    
    Bmatt = reshape(Bt,1+3*q,3);
            
    Bmatt = Bmatt';
            
    gammavec = Ct*Bmatt(:,1);
            
    Gamma3mat(1,1) = gammavec(3);
            
    for p = 1:q
                
        Gammamat = Ct*Bmatt(:,2+3*(p-1):4+3*(p-1));
                
        Gamma3mat(1,2+3*(p-1):4+3*(p-1)) = Gammamat(3,:);
                
    end
    
    xt = xdata(1,:);
    
    for t = 1:T
        
        Xpt = kron(eye(3),xt);
        
        Bt = BTmat(:,t);
        
        Ct = [ cppT(sweep,t) -cpgT(sweep,t) -cprT(sweep,t) ; ...
               -cgpT(sweep,t) cggT(sweep,t) -cgrT(sweep,t) ; ...
               -crpT(sweep,t) -crgT(sweep,t) crrT(sweep,t) ];
           
        Dt = diag([deltapT(sweep,t) deltagT(sweep,t) deltarT(sweep,t) ]);
        
        xit = [ xipT(sweep,t) ; xigT(sweep,t) ; xirT(sweep,t) ];
        
        if t > Tb-q-1
            
            Btildamatt = zeros(3,1+3*q);
            
            Ctildat = [ cppT(sweep,t) -cpgT(sweep,t) -cprT(sweep,t) ; ...
                        -cgpT(sweep,t) cggT(sweep,t) -cgrT(sweep,t) ; ...
                        -crpT(sweep,Tb-q) -crgT(sweep,Tb-q) crrT(sweep,Tb-q) ];
                    
            Bmatt = reshape(Bt,1+3*q,3);
            
            Bmatt = Bmatt';
            
            gammavec = Ct*Bmatt(:,1);
            gammavec(3) = Gamma3mat(1,1);
            muvec = Ctildat\gammavec;
            
            Btildamatt(:,1) = muvec;
            
            for p = 1:q
                
                Gammamat = Ct*Bmatt(:,2+3*(p-1):4+3*(p-1));
                Gammamat(3,:) = Gamma3mat(1,2+3*(p-1):4+3*(p-1));
                Btildamatt(:,2+3*(p-1):4+3*(p-1)) = Ctildat\Gammamat;
                
            end
            
            Bt = reshape(Btildamatt',3+9*q,1);
            Ct = Ctildat;
            Dt(3,3) = deltarT(sweep,Tb-q);
            
        end
        
        ut = Ct\(Dt*xit);
        
        yt = Xpt*Bt + ut;
        
        pdata4(sweep,t) = yt(1);
        gdata4(sweep,t) = yt(2);
        rdata4(sweep,t) = yt(3);
        
        xt(:,5:4+3*(q-1)) = xt(:,2:1+3*(q-1));
        xt(:,2:4) = yt';
    
    end
    
end

% =========================================================================
% Counterfactual with no monetary policy shocks over entire sample
% =========================================================================

for sweep = 1:nsweeps
    
    BTmat = betaT(:,:,sweep);
    
    xt = xdata(1,:);
    
    for t = 1:T
        
        Xpt = kron(eye(3),xt);
        
        Bt = BTmat(:,t);
        
        Ct = [ cppT(sweep,t) -cpgT(sweep,t) -cprT(sweep,t) ; ...
               -cgpT(sweep,t) cggT(sweep,t) -cgrT(sweep,t) ; ...
               -crpT(sweep,t) -crgT(sweep,t) crrT(sweep,t) ];
           
        Dt = diag([deltapT(sweep,t) deltagT(sweep,t) deltarT(sweep,t) ]);
        
        xit = [ xipT(sweep,t) ; xigT(sweep,t) ; 0 ];
        
        ut = Ct\(Dt*xit);
        
        yt = Xpt*Bt + ut;
        
        pdata5(sweep,t) = yt(1);
        gdata5(sweep,t) = yt(2);
        rdata5(sweep,t) = yt(3);
        
        xt(:,5:4+3*(q-1)) = xt(:,2:1+3*(q-1));
        xt(:,2:4) = yt';
    
    end
    
end

% =========================================================================
% Counterfactual with 1990 monetary policy rule over the entire sample
% period
% =========================================================================

for sweep = 1:nsweeps
    
    BTmat = betaT(:,:,sweep);
    
    Bt = BTmat(:,81-q);
    
    Ct = [ cppT(sweep,81-q) -cpgT(sweep,81-q) -cprT(sweep,81-q) ; ...
           -cgpT(sweep,81-q) cggT(sweep,81-q) -cgrT(sweep,81-q) ; ...
           -crpT(sweep,81-q) -crgT(sweep,81-q) crrT(sweep,81-q) ];
    
    Gamma3mat = zeros(1,1+3*q);
                    
    Bmatt = reshape(Bt,1+3*q,3);
            
    Bmatt = Bmatt';
            
    gammavec = Ct*Bmatt(:,1);
            
    Gamma3mat(1,1) = gammavec(3);
            
    for p = 1:q
                
        Gammamat = Ct*Bmatt(:,2+3*(p-1):4+3*(p-1));
                
        Gamma3mat(1,2+3*(p-1):4+3*(p-1)) = Gammamat(3,:);
                
    end
    
    xt = xdata(1,:);
    
    for t = 1:T
        
        Xpt = kron(eye(3),xt);
        
        Bt = BTmat(:,t);
        
        Ct = [ cppT(sweep,t) -cpgT(sweep,t) -cprT(sweep,t) ; ...
               -cgpT(sweep,t) cggT(sweep,t) -cgrT(sweep,t) ; ...
               -crpT(sweep,t) -crgT(sweep,t) crrT(sweep,t) ];
           
        Dt = diag([deltapT(sweep,t) deltagT(sweep,t) deltarT(sweep,t) ]);
        
        xit = [ xipT(sweep,t) ; xigT(sweep,t) ; xirT(sweep,t) ];
            
        Btildamatt = zeros(3,1+3*q);

        Ctildat = [ cppT(sweep,t) -cpgT(sweep,t) -cprT(sweep,t) ; ...
                    -cgpT(sweep,t) cggT(sweep,t) -cgrT(sweep,t) ; ...
                    -crpT(sweep,81-q) -crgT(sweep,81-q) crrT(sweep,81-q) ];

        Bmatt = reshape(Bt,1+3*q,3);

        Bmatt = Bmatt';

        gammavec = Ct*Bmatt(:,1);
        gammavec(3) = Gamma3mat(1,1);
        muvec = Ctildat\gammavec;

        Btildamatt(:,1) = muvec;

        for p = 1:q

            Gammamat = Ct*Bmatt(:,2+3*(p-1):4+3*(p-1));
            Gammamat(3,:) = Gamma3mat(1,2+3*(p-1):4+3*(p-1));
            Btildamatt(:,2+3*(p-1):4+3*(p-1)) = Ctildat\Gammamat;

        end

        Bt = reshape(Btildamatt',3+9*q,1);
        Ct = Ctildat;
            
        ut = Ct\(Dt*xit);
        
        yt = Xpt*Bt + ut;
        
        pdata6(sweep,t) = yt(1);
        gdata6(sweep,t) = yt(2);
        rdata6(sweep,t) = yt(3);
        
        xt(:,5:4+3*(q-1)) = xt(:,2:1+3*(q-1));
        xt(:,2:4) = yt';
    
    end
    
end

% =========================================================================
% Counterfactual with 1990 monetary policy rule from Tb forward
% =========================================================================

for sweep = 1:nsweeps
    
    BTmat = betaT(:,:,sweep);
    
    Bt = BTmat(:,81-q);
    
    Ct = [ cppT(sweep,81-q) -cpgT(sweep,81-q) -cprT(sweep,81-q) ; ...
           -cgpT(sweep,81-q) cggT(sweep,81-q) -cgrT(sweep,81-q) ; ...
           -crpT(sweep,81-q) -crgT(sweep,81-q) crrT(sweep,81-q) ];
    
    Gamma3mat = zeros(1,1+3*q);
                    
    Bmatt = reshape(Bt,1+3*q,3);
            
    Bmatt = Bmatt';
            
    gammavec = Ct*Bmatt(:,1);
            
    Gamma3mat(1,1) = gammavec(3);
            
    for p = 1:q
                
        Gammamat = Ct*Bmatt(:,2+3*(p-1):4+3*(p-1));
                
        Gamma3mat(1,2+3*(p-1):4+3*(p-1)) = Gammamat(3,:);
                
    end
    
    xt = xdata(1,:);
    
    for t = 1:T
        
        Xpt = kron(eye(3),xt);
        
        Bt = BTmat(:,t);
        
        Ct = [ cppT(sweep,t) -cpgT(sweep,t) -cprT(sweep,t) ; ...
               -cgpT(sweep,t) cggT(sweep,t) -cgrT(sweep,t) ; ...
               -crpT(sweep,t) -crgT(sweep,t) crrT(sweep,t) ];
           
        Dt = diag([deltapT(sweep,t) deltagT(sweep,t) deltarT(sweep,t) ]);
        
        xit = [ xipT(sweep,t) ; xigT(sweep,t) ; xirT(sweep,t) ];
        
        if t > Tb-q-1
            
            Btildamatt = zeros(3,1+3*q);
            
            Ctildat = [ cppT(sweep,t) -cpgT(sweep,t) -cprT(sweep,t) ; ...
                        -cgpT(sweep,t) cggT(sweep,t) -cgrT(sweep,t) ; ...
                        -crpT(sweep,81-q) -crgT(sweep,81-q) crrT(sweep,81-q) ];
                    
            Bmatt = reshape(Bt,1+3*q,3);
            
            Bmatt = Bmatt';
            
            gammavec = Ct*Bmatt(:,1);
            gammavec(3) = Gamma3mat(1,1);
            muvec = Ctildat\gammavec;
            
            Btildamatt(:,1) = muvec;
            
            for p = 1:q
                
                Gammamat = Ct*Bmatt(:,2+3*(p-1):4+3*(p-1));
                Gammamat(3,:) = Gamma3mat(1,2+3*(p-1):4+3*(p-1));
                Btildamatt(:,2+3*(p-1):4+3*(p-1)) = Ctildat\Gammamat;
                
            end
            
            Bt = reshape(Btildamatt',3+9*q,1);
            Ct = Ctildat;
            
        end
        
        ut = Ct\(Dt*xit);
        
        yt = Xpt*Bt + ut;
        
        pdata7(sweep,t) = yt(1);
        gdata7(sweep,t) = yt(2);
        rdata7(sweep,t) = yt(3);
        
        xt(:,5:4+3*(q-1)) = xt(:,2:1+3*(q-1));
        xt(:,2:4) = yt';
    
    end
    
end

% =========================================================================
% Save results
% =========================================================================

clear betaT;
save('tvpvarout.mat');

