% tvpvarsim: Simulates the posterior kernel for a three-variable VAR with
% time-varying parameters and stochastic volatility using the Gibbs
% sampling algorithm outlined by Primiceri (2005) and corrected by Del
% Negro and Primiceri (2015). The VAR includes inflation, a gap variable
% (the output gap or unemployment rate), and the short-term nominal
% interest rate. The structural parameters and disturbances can be
% recovered from the reduced form using either a triangular ordering or a
% scheme based on sign restrictions.

% =========================================================================
% These programs were written for MATLAB R2016a by Michael Belongia and
% Peter Ireland and used to generate the results in our paper, "The
% Evolution of U.S. Monetary Policy: 2000-2007," which appears in the
% December 2016 issue of the Journal of Economic Dynamics and Control.
% =========================================================================

% =========================================================================
% Set global variables, load data, choose variables and VAR lag length,
% choose identification scheme, decide whether or not to discard
% coefficient draws that imply explosive VAR dynamics, and select the
% number of sweeps for the burn-in and Gibbs sampling phases, and decide
% whether to use the Kim-Shephard-Chib approximation to the posterior or a
% Metropolis-Hastings step from drawing volatility states.
% =========================================================================

clear;
rng('default');

global ydata xdata T q Ra ra
global muB scaleB muA scaleA mulSig scalelSig 
global qvec mvec v2vec
global sigMH sigaccept sigreject

load tvpvardata.dat

% Quarterly data file has 229 rows, covering 1959:1 through 2015:4, with
% columns:
%     1 - Core PCE Price Index
%     2 - Core CPI
%     3 - Real GDP
%     4 - Potential GDP
%     5 - Unemployment Rate
%     6 - Federal Funds Rate
% Data sample runs from 1960:1 through 2007:4, with 1960:1-1969:4 used as a
% training sample for calibrating the priors and 1970:1-2007:4 used to
% estimate the TVP-VAR.

corepceinfyoy = 100*(log(tvpvardata(5:196,1))-log(tvpvardata(1:192,1)));
corecpiinfyoy = 100*(log(tvpvardata(5:196,2))-log(tvpvardata(1:192,2)));
ygap = 100*(log(tvpvardata(5:196,3))-log(tvpvardata(5:196,4)));
unemp = tvpvardata(5:196,5);
rff = tvpvardata(5:196,6);

tsdata = [ corepceinfyoy(1:40) ygap(1:40) rff(1:40) ]; 
vardata = [ corepceinfyoy(41:192) ygap(41:192) rff(41:192) ];
gsign = 1;
% estimate with core PCE inflation, output gap

% tsdata = [ corecpiinfyoy(1:40) unemp(1:40) rff(1:40) ]; 
% vardata = [ corecpiinfyoy(41:192) unemp(41:192) rff(41:192) ];
% gsign = -1;
% estimate with core CPI inflation, unemployment rate

% gsign is used to identify shocks using sign restrictions
% set gsign=1 if using the output gap
% set gsign=-1 if using the unemployment rate

q = 2;
% VAR lag length

T = length(vardata(:,1)) - q;

ydata = vardata(q+1:T+q,:);
xdata = [ ones(T,1) zeros(T,3*q) ];
for p = 1:q
    xdata(:,2+3*(p-1):4+3*(p-1)) = vardata(q+1-p:T+q-p,:);
end

idscheme = 2;
% 1=Triangular, 2=Sign restrictions

discard = 1;
% 1=Discard explosive draws, 0=Do not discard explosive draws

nburnins = 100000;
nsweeps = 50000;
thinfactor = 1;
nstores = nsweeps/thinfactor;
% length of posterior simulation

Baccept = 0;
Breject = 0;
% keep track of explosive coefficient acceptance/rejection rate

sigaccept = 0;
sigreject = 0;
% keep track of volatility acceptance/rejection rate

Signaccept = 0;
Signreject = 0;
% keep track of sign restriction acceptance/rejection rate

maxBdraws = 100;
MaxBacksteps = 25;
Backsteps = 0;
% parameters and counter for backsteps if explosive draws are discarded

sigMH = 1;
% sigMH=0 to use Kim-Shephard-Chib approximation to posterior
% sigMH=1 to use Metropolis-Hastings step for drawing volatilty states

qvec = [ 0.00730 ; 0.10556 ; 0.00002 ; 0.04395 ; 0.34001 ; 0.24566 ; 0.25750 ];
mvec = [ -10.12999 ; -3.97281 ; -8.56686 ; 2.77786 ; 0.61942 ; 1.79518 ; -1.08819 ];
v2vec = [ 5.79596 ; 2.61369 ; 5.17950 ; 0.16735; 0.64009 ; 0.34023 ; 1.26261 ];
% parameters for Kim-Shephard-Chib algorithm

Ra = zeros(9,3);
Ra(2,1) = 1;
Ra(3,2) = 1;
Ra(6,3) = 1;
ra = zeros(9,1);
ra(1) = 1;
ra(5) = 1;
ra(9) = 1;
% matrices for converting the vector alpha into the matrix A

% =========================================================================
% Set up priors
% =========================================================================

[ Bhat,VB,alphahat,Valpha,sigmahat,Vsigma ] = cpvarest(tsdata);
    
muB = Bhat;
scaleB = 4*VB;
    
muA = alphahat;
scaleA = 4*Valpha;
    
mulSig = log(sigmahat);
scalelSig = eye(3);

kQ = sqrt(0.00035);
kS = 0.1;
kW = 0.01;

dfQ = 22;
scaleQ = (kQ^2)*dfQ*VB;

dfS1 = 2;
scaleS1 = (kS^2)*dfS1*Valpha(1,1);

dfS2 = 3;
scaleS2 = (kS^2)*dfS2*Valpha(2:3,2:3);

scaleS = [ scaleS1 zeros(1,2) ; zeros(2,1) scaleS2 ];
    
dfW = 2;
scaleW = (kW^2)*dfW*eye(3);
    
% =========================================================================
% Set up output matrices
% =========================================================================

betaT = zeros(3+9*q,T,nstores);

alphagpT = zeros(nstores,T);
alpharpT = zeros(nstores,T);
alphargT = zeros(nstores,T);

sigpT = zeros(nstores,T);
siggT = zeros(nstores,T);
sigrT = zeros(nstores,T);

QT = zeros((3+9*q)*(3+9*q+1)/2,nstores);
S1T = zeros(1,nstores);
S2T = zeros(3,nstores);
WT = zeros(3,nstores);

mupT = zeros(nstores,T);
mugT = zeros(nstores,T);
murT = zeros(nstores,T);

cppT = zeros(nstores,T);
cpgT = zeros(nstores,T);
cprT = zeros(nstores,T);
cgpT = zeros(nstores,T);
cggT = zeros(nstores,T);
cgrT = zeros(nstores,T);
crpT = zeros(nstores,T);
crgT = zeros(nstores,T);
crrT = zeros(nstores,T);

deltapT = zeros(nstores,T);
deltagT = zeros(nstores,T);
deltarT = zeros(nstores,T);

gammarrT = zeros(nstores,T);
gammarpT = zeros(nstores,T);
gammargT = zeros(nstores,T);

xipT = zeros(nstores,T);
xigT = zeros(nstores,T);
xirT = zeros(nstores,T);

psimp75 = zeros(nstores,41);
gsimp75 = zeros(nstores,41);
rsimp75 = zeros(nstores,41);

pdimp75 = zeros(nstores,41);
gdimp75 = zeros(nstores,41);
rdimp75 = zeros(nstores,41);

pmimp75 = zeros(nstores,41);
gmimp75 = zeros(nstores,41);
rmimp75 = zeros(nstores,41);

psimp90 = zeros(nstores,41);
gsimp90 = zeros(nstores,41);
rsimp90 = zeros(nstores,41);

pdimp90 = zeros(nstores,41);
gdimp90 = zeros(nstores,41);
rdimp90 = zeros(nstores,41);

pmimp90 = zeros(nstores,41);
gmimp90 = zeros(nstores,41);
rmimp90 = zeros(nstores,41);

psimp00 = zeros(nstores,41);
gsimp00 = zeros(nstores,41);
rsimp00 = zeros(nstores,41);

pdimp00 = zeros(nstores,41);
gdimp00 = zeros(nstores,41);
rdimp00 = zeros(nstores,41);

pmimp00 = zeros(nstores,41);
gmimp00 = zeros(nstores,41);
rmimp00 = zeros(nstores,41);

psimp07 = zeros(nstores,41);
gsimp07 = zeros(nstores,41);
rsimp07 = zeros(nstores,41);

pdimp07 = zeros(nstores,41);
gdimp07 = zeros(nstores,41);
rdimp07 = zeros(nstores,41);

pmimp07 = zeros(nstores,41);
gmimp07 = zeros(nstores,41);
rmimp07 = zeros(nstores,41);

psvdec75 = zeros(nstores,40);
gsvdec75 = zeros(nstores,40);
rsvdec75 = zeros(nstores,40);

pdvdec75 = zeros(nstores,40);
gdvdec75 = zeros(nstores,40);
rdvdec75 = zeros(nstores,40);

pmvdec75 = zeros(nstores,40);
gmvdec75 = zeros(nstores,40);
rmvdec75 = zeros(nstores,40);

psvdec90 = zeros(nstores,40);
gsvdec90 = zeros(nstores,40);
rsvdec90 = zeros(nstores,40);

pdvdec90 = zeros(nstores,40);
gdvdec90 = zeros(nstores,40);
rdvdec90 = zeros(nstores,40);

pmvdec90 = zeros(nstores,40);
gmvdec90 = zeros(nstores,40);
rmvdec90 = zeros(nstores,40);

psvdec00 = zeros(nstores,40);
gsvdec00 = zeros(nstores,40);
rsvdec00 = zeros(nstores,40);

pdvdec00 = zeros(nstores,40);
gdvdec00 = zeros(nstores,40);
rdvdec00 = zeros(nstores,40);

pmvdec00 = zeros(nstores,40);
gmvdec00 = zeros(nstores,40);
rmvdec00 = zeros(nstores,40);

psvdec07 = zeros(nstores,40);
gsvdec07 = zeros(nstores,40);
rsvdec07 = zeros(nstores,40);

pdvdec07 = zeros(nstores,40);
gdvdec07 = zeros(nstores,40);
rdvdec07 = zeros(nstores,40);

pmvdec07 = zeros(nstores,40);
gmvdec07 = zeros(nstores,40);
rmvdec07 = zeros(nstores,40);

if discard == 1
    
    BTback = zeros(3+9*q,T,MaxBacksteps);
    ATback = zeros(3,T,MaxBacksteps);
    lSigTback = zeros(3,T,MaxBacksteps);
    Qback = zeros(3+9*q,3+9*q,MaxBacksteps);
    Sback = zeros(3,3,MaxBacksteps);
    Wback = zeros(3,3,MaxBacksteps);
    AvailBacksteps = 0;
    
end

% =========================================================================
% Burn-in for Gibbs Sampler
% =========================================================================

bisweep = 1;

while bisweep <= nburnins

	Bnotdone = 1;

    while Bnotdone
    
        if bisweep == 1
        	initstates;
        end
    
        if bisweep/100 == round(bisweep/100)
        	fprintf('burn-in phase: sweep = %d\r',bisweep);
            fprintf('Acceptance rate for coefficient draws = %.2f\r',Baccept/max([Baccept+Breject 1]));
            fprintf('Acceptance rate for volatility draws = %.2f\r',sigaccept/max([sigaccept+sigreject 1]));
            fprintf('Acceptance rate for sign restrictions = %.2f\r',Signaccept/max([Signaccept+Signreject 1]));
        end
        
        BdrawNum = 1;
        
        while Bnotdone && (BdrawNum <= maxBdraws)
    
            BTdraw = Bdraw(ATmat,lSigTmat,Q); 
        
            if discard == 1
            
                noeigviol = 1;
                t = 1;
            
                while noeigviol && (t <= T)
                
                    Bmatt = reshape(BTdraw(:,t),1+3*q,3);
                      
                    BigBt = [ Bmatt(2:1+3*q,:)' ; eye(3*(q-1)) zeros(3*(q-1),3) ];
            
                    if max(abs(eig(BigBt))) <= 1
                
                        t = t + 1;
                        
                    else
                        
                        noeigviol = 0;
                  
                    end
            
                end
            
                if noeigviol
                
                    Bnotdone = 0;
                    Baccept = Baccept + 1;
                
                else
                
                    BdrawNum = BdrawNum + 1;
                    Breject = Breject + 1;
                
                end
            
            else
            
                Bnotdone = 0;
                Baccept = Baccept + 1;
            
            end
            
        end
        
        if Bnotdone
            
            if AvailBacksteps > 0
                
                BTmat = BTback(:,:,1);
                ATmat = ATback(:,:,1);
                lSigTmat = lSigTback(:,:,1);
                Q = Qback(:,:,1);
                S = Sback(:,:,1);
                S1 = S(1,1); S2 = S(2:3,2:3);
                W = Wback(:,:,1);
                
                BTback(:,:,1:MaxBacksteps-1) = BTback(:,:,2:MaxBacksteps);
                BTback(:,:,MaxBacksteps) = zeros(3+9*q,T);
        
                ATback(:,:,1:MaxBacksteps-1) = ATback(:,:,2:MaxBacksteps);
                ATback(:,:,MaxBacksteps) = zeros(3,T);
                
                lSigTback(:,:,1:MaxBacksteps-1) = lSigTback(:,:,2:MaxBacksteps);
                lSigTback(:,:,MaxBacksteps) = zeros(3,T);
                
                Qback(:,:,1:MaxBacksteps-1) = Qback(:,:,2:MaxBacksteps);
                Qback(:,:,MaxBacksteps) = zeros(3+9*q,3+9*q);
                
                Sback(:,:,1:MaxBacksteps-1) = Sback(:,:,2:MaxBacksteps);
                Sback(:,:,MaxBacksteps) = zeros(3,3);
                
                Wback(:,:,1:MaxBacksteps-1) = Wback(:,:,2:MaxBacksteps);
                Wback(:,:,MaxBacksteps) = zeros(3,3);
                
                Backsteps = Backsteps + 1;
                AvailBacksteps = AvailBacksteps - 1;
                bisweep = max([ bisweep-1 1 ]);
                
            end
            
        else
            
            BTmat = BTdraw;
            
            if idscheme == 1
                
                ATmat = Adraw(BTmat,lSigTmat,S);
    
                lSigTmat = lSigdraw(BTmat,ATmat,lSigTmat,W);
                
                Signaccept = Signaccept + 1;
                
            else
                
                ASnotdone = 1;
                
                while ASnotdone
                
                    ATdraw = Adraw(BTmat,lSigTmat,S);

                    lSigTdraw = lSigdraw(BTmat,ATdraw,lSigTmat,W);
                    
                    Xtilde = randn(3);
                    [ Qx,Rx ] = qr(Xtilde);
                    for j = 1:3
                        if Rx(j,j) < 0
                            Qx(:,j) = -Qx(:,j);
                        end
                    end
                    
                    nosignviol = 1;
                    t = 1;
                    
                    while nosignviol && (t <= T)
                        
                        Avect = Ra*ATdraw(:,t) + ra;
                        Amatt = reshape(Avect,3,3);
                        
                        Sigmatt = diag(exp(lSigTdraw(:,t)));
                        
                        Lt = Amatt\Sigmatt;
                        
                        Ltildat = Lt*Qx';
                        
                        signchecks = -gsign*Ltildat(1,1)*Ltildat(2,1);
                        
                        signcheckd1 = gsign*Ltildat(1,2)*Ltildat(2,2);
                        signcheckd2 = gsign*Ltildat(2,2)*Ltildat(3,2);
                        
                        signcheckm1 = -Ltildat(1,3)*Ltildat(3,3);
                        signcheckm2 = -gsign*Ltildat(2,3)*Ltildat(3,3);
                        
                        if (signchecks>0) && (signcheckd1>0) && (signcheckd2>0) && (signcheckm1>0) && (signcheckm2>0)
                            t = t + 1;
                        else
                            nosignviol = 0;
                        end
                        
                    end
                    
                    if nosignviol
                        
                        ATmat = ATdraw;
                        lSigTmat = lSigTdraw;
                        
                        ASnotdone = 0;
                        Signaccept = Signaccept + 1;
                        
                    else
                        
                        Signreject = Signreject + 1;
                        
                    end
                
                end
                
            end
    
            nuTmat = BTmat(:,2:T) - BTmat(:,1:T-1);
    
            zetaTmat = ATmat(:,2:T) - ATmat(:,1:T-1);
    
            etaTmat = lSigTmat(:,2:T) - lSigTmat(:,1:T-1);
    
            SigmaB = scaleQ;
    
            SigmaA1 = scaleS1;
            
            SigmaA2 = scaleS2;
    
            SigmaSig = scaleW;
    
            for t = 1:T-1
        
                SigmaB = SigmaB + nuTmat(:,t)*nuTmat(:,t)';
        
                SigmaA1 = SigmaA1 + zetaTmat(1,t)*zetaTmat(1,t)';
                
                SigmaA2 = SigmaA2 + zetaTmat(2:3,t)*zetaTmat(2:3,t)';
        
                SigmaSig = SigmaSig + diag(diag(etaTmat(:,t)*etaTmat(:,t)'));
        
            end
    
            Q = invwishdraw(SigmaB,dfQ+T);
    
            S1 = invwishdraw(SigmaA1,dfS1+T);
            
            S2 = invwishdraw(SigmaA2,dfS2+T);
            
            S = [ S1 zeros(1,2) ; zeros(2,1) S2 ];
       
            W11 = invwishdraw(SigmaSig(1,1),dfW+T);
    
            W22 = invwishdraw(SigmaSig(2,2),dfW+T);
    
            W33 = invwishdraw(SigmaSig(3,3),dfW+T);
    
            W = diag([ W11 W22 W33 ]);
            
            if discard == 1
                
                BTback(:,:,2:MaxBacksteps) = BTback(:,:,1:MaxBacksteps-1);
                BTback(:,:,1) = BTmat;
        
                ATback(:,:,2:MaxBacksteps) = ATback(:,:,1:MaxBacksteps-1);
                ATback(:,:,1) = ATmat;
                
                lSigTback(:,:,2:MaxBacksteps) = lSigTback(:,:,1:MaxBacksteps-1);
                lSigTback(:,:,1) = lSigTmat;
                
                Qback(:,:,2:MaxBacksteps) = Qback(:,:,1:MaxBacksteps-1);
                Qback(:,:,1) = Q;
                
                Sback(:,:,2:MaxBacksteps) = Sback(:,:,1:MaxBacksteps-1);
                Sback(:,:,1) = S;
                
                Wback(:,:,2:MaxBacksteps) = Wback(:,:,1:MaxBacksteps-1);
                Wback(:,:,1) = W;     
                
                AvailBacksteps = min([ AvailBacksteps+1 MaxBacksteps ]);
                
            end
            
            bisweep = bisweep + 1;
    
        end
        
    end
    
end

Baccept_burnin = Baccept;
Breject_burnin = Breject;

sigaccept_burnin = sigaccept;
sigreject_burnin = sigreject;

Signaccept_burnin = Signaccept;
Signreject_burnin = Signreject;

% =========================================================================
% Gibbs Sampler
% =========================================================================

sweep = 1;

while sweep <= nsweeps
        
    Bnotdone = 1;
    
    if sweep/100 == round(sweep/100)
        fprintf('Gibbs sampling phase: sweep = %d\r',sweep);
        fprintf('Acceptance rate for coefficient draws = %.2f\r',Baccept/max([Baccept+Breject 1]));
        fprintf('Acceptance rate for volatility draws = %.2f\r',sigaccept/max([sigaccept+sigreject 1]));
        fprintf('Acceptance rate for sign restrictions = %.2f\r',Signaccept/max([Signaccept+Signreject 1]));
    end
    
    while Bnotdone
        
        BdrawNum = 1;
        
        while Bnotdone && (BdrawNum <= maxBdraws)
            
            BTdraw = Bdraw(ATmat,lSigTmat,Q); 
        
            if discard == 1
            
                noeigviol = 1;
                t = 1;
            
                while noeigviol && (t <= T)
                
                    Bmatt = reshape(BTdraw(:,t),1+3*q,3);
                      
                    BigBt = [ Bmatt(2:1+3*q,:)' ; eye(3*(q-1)) zeros(3*(q-1),3) ];
            
                    if max(abs(eig(BigBt))) <= 1
                
                        t = t + 1;
                        
                    else
                        
                        noeigviol = 0;
                  
                    end
            
                end
            
                if noeigviol
                
                    Bnotdone = 0;
                    Baccept = Baccept + 1;
                
                else
                
                    BdrawNum = BdrawNum + 1;
                    Breject = Breject + 1;
                
                end
            
            else
            
                Bnotdone = 0;
                Baccept = Baccept + 1;
            
            end
            
        end
        
        if Bnotdone
            
            if AvailBacksteps > 0
                
                BTmat = BTback(:,:,1);
                ATmat = ATback(:,:,1);
                lSigTmat = lSigTback(:,:,1);
                Q = Qback(:,:,1);
                S = Sback(:,:,1);
                S1 = S(1,1); S2 = S(2:3,2:3);
                W = Wback(:,:,1);
                
                BTback(:,:,1:MaxBacksteps-1) = BTback(:,:,2:MaxBacksteps);
                BTback(:,:,MaxBacksteps) = zeros(3+9*q,T);
        
                ATback(:,:,1:MaxBacksteps-1) = ATback(:,:,2:MaxBacksteps);
                ATback(:,:,MaxBacksteps) = zeros(3,T);
                
                lSigTback(:,:,1:MaxBacksteps-1) = lSigTback(:,:,2:MaxBacksteps);
                lSigTback(:,:,MaxBacksteps) = zeros(3,T);
                
                Qback(:,:,1:MaxBacksteps-1) = Qback(:,:,2:MaxBacksteps);
                Qback(:,:,MaxBacksteps) = zeros(3+9*q,3+9*q);
                
                Sback(:,:,1:MaxBacksteps-1) = Sback(:,:,2:MaxBacksteps);
                Sback(:,:,MaxBacksteps) = zeros(3,3);
                
                Wback(:,:,1:MaxBacksteps-1) = Wback(:,:,2:MaxBacksteps);
                Wback(:,:,MaxBacksteps) = zeros(3,3);
                
                Backsteps = Backsteps + 1;
                AvailBacksteps = AvailBacksteps - 1;
                sweep = sweep - 1;
                
            end
            
        else
            
            BTmat = BTdraw;
            
            if idscheme == 1
            
                ATmat = Adraw(BTmat,lSigTmat,S);
    
                lSigTmat = lSigdraw(BTmat,ATmat,lSigTmat,W);
                
                Signaccept = Signaccept + 1;
                
            else
                
                ASnotdone = 1;
                
                while ASnotdone
                    
                    ATdraw = Adraw(BTmat,lSigTmat,S);
                    
                    lSigTdraw = lSigdraw(BTmat,ATdraw,lSigTmat,W);
                    
                    Xtilde = randn(3);
                    [ Qx,Rx ] = qr(Xtilde);
                    for j = 1:3
                        if Rx(j,j) < 0
                            Qx(:,j) = -Qx(:,j);
                        end
                    end
                    
                    nosignviol = 1;
                    t = 1;
                    
                    while nosignviol && (t <= T)
                        
                        Avect = Ra*ATdraw(:,t) + ra;
                        Amatt = reshape(Avect,3,3);
                        
                        Sigmatt = diag(exp(lSigTdraw(:,t)));
                        
                        Lt = Amatt\Sigmatt;
                        
                        Ltildat = Lt*Qx';
                        
                        signchecks = -gsign*Ltildat(1,1)*Ltildat(2,1);
                        
                        signcheckd1 = gsign*Ltildat(1,2)*Ltildat(2,2);
                        signcheckd2 = gsign*Ltildat(2,2)*Ltildat(3,2);
                        
                        signcheckm1 = -Ltildat(1,3)*Ltildat(3,3);
                        signcheckm2 = -gsign*Ltildat(2,3)*Ltildat(3,3);
                        
                        if (signchecks>0) && (signcheckd1>0) && (signcheckd2>0) && (signcheckm1>0) && (signcheckm2>0)
                            t = t + 1;
                        else
                            nosignviol = 0;
                        end
                        
                    end
                    
                    if nosignviol
                        
                        ATmat = ATdraw;
                        lSigTmat = lSigTdraw;
                        
                        ASnotdone = 0;
                        Signaccept = Signaccept + 1;
                        
                    else
                        
                        Signreject = Signreject + 1;
                        
                    end
                
                end
                
            end  
          
            nuTmat = BTmat(:,2:T) - BTmat(:,1:T-1);
    
            zetaTmat = ATmat(:,2:T) - ATmat(:,1:T-1);
    
            etaTmat = lSigTmat(:,2:T) - lSigTmat(:,1:T-1);
    
            SigmaB = scaleQ;
    
            SigmaA1 = scaleS1;
            
            SigmaA2 = scaleS2;
    
            SigmaSig = scaleW;
    
            for t = 1:T-1
        
                SigmaB = SigmaB + nuTmat(:,t)*nuTmat(:,t)';
        
                SigmaA1 = SigmaA1 + zetaTmat(1,t)*zetaTmat(1,t)';
                
                SigmaA2 = SigmaA2 + zetaTmat(2:3,t)*zetaTmat(2:3,t)';
        
                SigmaSig = SigmaSig + diag(diag(etaTmat(:,t)*etaTmat(:,t)'));
        
            end
    
            Q = invwishdraw(SigmaB,dfQ+T);
    
            S1 = invwishdraw(SigmaA1,dfS1+T);
            
            S2 = invwishdraw(SigmaA2,dfS2+T);
            
            S = [ S1 zeros(1,2) ; zeros(2,1) S2 ];
       
            W11 = invwishdraw(SigmaSig(1,1),dfW+T);
    
            W22 = invwishdraw(SigmaSig(2,2),dfW+T);
    
            W33 = invwishdraw(SigmaSig(3,3),dfW+T);
    
            W = diag([ W11 W22 W33 ]);
            
            if sweep/thinfactor == round(sweep/thinfactor)
                
                storenum = sweep/thinfactor;
    
                betaT(:,:,storenum) = BTmat;
    
                alphagpT(storenum,:) = ATmat(1,:);
                alpharpT(storenum,:) = ATmat(2,:);
                alphargT(storenum,:) = ATmat(3,:);

                sigpT(storenum,:) = exp(lSigTmat(1,:));
                siggT(storenum,:) = exp(lSigTmat(2,:));
                sigrT(storenum,:) = exp(lSigTmat(3,:));
    
                QT(:,storenum) = nonzeros(tril(Q));
                S1T(:,storenum) = S(1,1);
                S2T(:,storenum) = [ S(2,2) ; S(3,2) ; S(3,3) ];
                WT(:,storenum) = diag(W);
                
                if idscheme == 1
                    
                    cppT(storenum,:) = ones(1,T);
                    cgpT(storenum,:) = -alphagpT(storenum,:);
                    cggT(storenum,:) = ones(1,T);
                    crpT(storenum,:) = -alpharpT(storenum,:);
                    crgT(storenum,:) = -alphargT(storenum,:);
                    crrT(storenum,:) = ones(1,T);

                    deltapT(storenum,:) = sigpT(storenum,:);
                    deltagT(storenum,:) = siggT(storenum,:);
                    deltarT(storenum,:) = sigrT(storenum,:);
                    
                end
                
                for t = 1:T
                    
                    Bmatt = reshape(BTmat(:,t),1+3*q,3);
                    mut = [ Bmatt(1,:)' ; zeros(3*(q-1),1) ];
                    BigBt = [ Bmatt(2:1+3*q,:)' ; eye(3*(q-1)) zeros(3*(q-1),3) ];
                    mubart = (eye(3*q)-BigBt)\mut;
                    mupT(storenum,t) = mubart(1);
                    mugT(storenum,t) = mubart(2);
                    murT(storenum,t) = mubart(3);
                    
                    Avect = Ra*ATmat(:,t) + ra;
                    Amatt = reshape(Avect,3,3);
                    
                    Sigmatt = diag([ sigpT(storenum,t) siggT(storenum,t) sigrT(storenum,t) ]);
                    
                    if idscheme == 1
                        
                        Cmatt = Amatt;
                        
                        Dmatt = Sigmatt;
                        
                    else
                        
                        Lt = Amatt\Sigmatt;
                        
                        Ltildat = Lt*Qx';
                        
                        for j = 1:3
                            if Ltildat(j,j) < 0
                                Ltildat(:,j) = -Ltildat(:,j);
                            end
                        end
                        
                        Linvt = eye(3)/Ltildat;
                        
                        Dinvt = diag(abs(diag(Linvt)));
                        Dmatt = eye(3)/Dinvt;
                        
                        for j = 1:3
                            Linvt(j,:) = Linvt(j,:)/abs(Linvt(j,j));
                        end
                        
                        Cmatt = Linvt;
                        
                        cppT(storenum,t) = Cmatt(1,1);
                        cpgT(storenum,t) = -Cmatt(1,2);
                        cprT(storenum,t) = -Cmatt(1,3);
                        cgpT(storenum,t) = -Cmatt(2,1);
                        cggT(storenum,t) = Cmatt(2,2);
                        cgrT(storenum,t) = -Cmatt(2,3);
                        crpT(storenum,t) = -Cmatt(3,1);
                        crgT(storenum,t) = -Cmatt(3,2);
                        crrT(storenum,t) = Cmatt(3,3);

                        deltapT(storenum,t) = Dmatt(1,1);
                        deltagT(storenum,t) = Dmatt(2,2);
                        deltarT(storenum,t) = Dmatt(3,3);
                        
                    end
                    
                    gammarpT(storenum,t) = crpT(storenum,t);
                    gammargT(storenum,t) = crgT(storenum,t);
                    gammarrT(storenum,t) = 0;
                    
                    for p = 1:q
                        
                        Gammaqt = Cmatt*Bmatt(2+3*(p-1):4+3*(p-1),:)';
                        gammarpT(storenum,t) = gammarpT(storenum,t) + Gammaqt(3,1);
                        gammargT(storenum,t) = gammargT(storenum,t) + Gammaqt(3,2);
                        gammarrT(storenum,t) = gammarrT(storenum,t) + Gammaqt(3,3);
                        
                    end
                    
                    gammarpT(storenum,t) = gammarpT(storenum,t)/(crrT(storenum,t)-gammarrT(storenum,t));
                    gammargT(storenum,t) = gammargT(storenum,t)/(crrT(storenum,t)-gammarrT(storenum,t));
                    
                    
                    yt = ydata(t,:)';
                    Xpt = kron(eye(3),xdata(t,:));
                    ut = yt - Xpt*BTmat(:,t);
                    xit = Dmatt\(Cmatt*ut);
                    xipT(storenum,t) = xit(1);
                    xigT(storenum,t) = xit(2);
                    xirT(storenum,t) = xit(3);
                    
                    if t == 21 - q
                        
                        imps = impulses(BTmat(:,t),Cmatt,Dmatt,1);
                        psimp75(storenum,:) = imps(:,1)';
                        gsimp75(storenum,:) = imps(:,2)';
                        rsimp75(storenum,:) = imps(:,3)';
                        
                        impd = impulses(BTmat(:,t),Cmatt,Dmatt,2);
                        pdimp75(storenum,:) = impd(:,1)';
                        gdimp75(storenum,:) = impd(:,2)';
                        rdimp75(storenum,:) = impd(:,3)';
                        
                        impm = impulses(BTmat(:,t),Cmatt,Dmatt,3);
                        pmimp75(storenum,:) = impm(:,1)';
                        gmimp75(storenum,:) = impm(:,2)';
                        rmimp75(storenum,:) = impm(:,3)';
                        
                        vdecs = vardecomp(BTmat(:,t),Cmatt,Dmatt,1);
                        psvdec75(storenum,:) = vdecs(:,1)';
                        gsvdec75(storenum,:) = vdecs(:,2)';
                        rsvdec75(storenum,:) = vdecs(:,3)';
                        
                        vdecd = vardecomp(BTmat(:,t),Cmatt,Dmatt,2);
                        pdvdec75(storenum,:) = vdecd(:,1)';
                        gdvdec75(storenum,:) = vdecd(:,2)';
                        rdvdec75(storenum,:) = vdecd(:,3)';
                        
                        vdecm = vardecomp(BTmat(:,t),Cmatt,Dmatt,3);
                        pmvdec75(storenum,:) = vdecm(:,1)';
                        gmvdec75(storenum,:) = vdecm(:,2)';
                        rmvdec75(storenum,:) = vdecm(:,3)';
              
                    end
                    
                    if t == 81 - q
                        
                        imps = impulses(BTmat(:,t),Cmatt,Dmatt,1);
                        psimp90(storenum,:) = imps(:,1)';
                        gsimp90(storenum,:) = imps(:,2)';
                        rsimp90(storenum,:) = imps(:,3)';
                        
                        impd = impulses(BTmat(:,t),Cmatt,Dmatt,2);
                        pdimp90(storenum,:) = impd(:,1)';
                        gdimp90(storenum,:) = impd(:,2)';
                        rdimp90(storenum,:) = impd(:,3)';
                        
                        impm = impulses(BTmat(:,t),Cmatt,Dmatt,3);
                        pmimp90(storenum,:) = impm(:,1)';
                        gmimp90(storenum,:) = impm(:,2)';
                        rmimp90(storenum,:) = impm(:,3)';
                        
                        vdecs = vardecomp(BTmat(:,t),Cmatt,Dmatt,1);
                        psvdec90(storenum,:) = vdecs(:,1)';
                        gsvdec90(storenum,:) = vdecs(:,2)';
                        rsvdec90(storenum,:) = vdecs(:,3)';
                        
                        vdecd = vardecomp(BTmat(:,t),Cmatt,Dmatt,2);
                        pdvdec90(storenum,:) = vdecd(:,1)';
                        gdvdec90(storenum,:) = vdecd(:,2)';
                        rdvdec90(storenum,:) = vdecd(:,3)';
                        
                        vdecm = vardecomp(BTmat(:,t),Cmatt,Dmatt,3);
                        pmvdec90(storenum,:) = vdecm(:,1)';
                        gmvdec90(storenum,:) = vdecm(:,2)';
                        rmvdec90(storenum,:) = vdecm(:,3)';
              
                    end
                    
                    if t == 121 - q
                        
                        imps = impulses(BTmat(:,t),Cmatt,Dmatt,1);
                        psimp00(storenum,:) = imps(:,1)';
                        gsimp00(storenum,:) = imps(:,2)';
                        rsimp00(storenum,:) = imps(:,3)';
                        
                        impd = impulses(BTmat(:,t),Cmatt,Dmatt,2);
                        pdimp00(storenum,:) = impd(:,1)';
                        gdimp00(storenum,:) = impd(:,2)';
                        rdimp00(storenum,:) = impd(:,3)';
                        
                        impm = impulses(BTmat(:,t),Cmatt,Dmatt,3);
                        pmimp00(storenum,:) = impm(:,1)';
                        gmimp00(storenum,:) = impm(:,2)';
                        rmimp00(storenum,:) = impm(:,3)';
                        
                        vdecs = vardecomp(BTmat(:,t),Cmatt,Dmatt,1);
                        psvdec00(storenum,:) = vdecs(:,1)';
                        gsvdec00(storenum,:) = vdecs(:,2)';
                        rsvdec00(storenum,:) = vdecs(:,3)';
                        
                        vdecd = vardecomp(BTmat(:,t),Cmatt,Dmatt,2);
                        pdvdec00(storenum,:) = vdecd(:,1)';
                        gdvdec00(storenum,:) = vdecd(:,2)';
                        rdvdec00(storenum,:) = vdecd(:,3)';
                        
                        vdecm = vardecomp(BTmat(:,t),Cmatt,Dmatt,3);
                        pmvdec00(storenum,:) = vdecm(:,1)';
                        gmvdec00(storenum,:) = vdecm(:,2)';
                        rmvdec00(storenum,:) = vdecm(:,3)';
              
                    end
                    
                    if t == 152 - q
                        
                        imps = impulses(BTmat(:,t),Cmatt,Dmatt,1);
                        psimp07(storenum,:) = imps(:,1)';
                        gsimp07(storenum,:) = imps(:,2)';
                        rsimp07(storenum,:) = imps(:,3)';
                        
                        impd = impulses(BTmat(:,t),Cmatt,Dmatt,2);
                        pdimp07(storenum,:) = impd(:,1)';
                        gdimp07(storenum,:) = impd(:,2)';
                        rdimp07(storenum,:) = impd(:,3)';
                        
                        impm = impulses(BTmat(:,t),Cmatt,Dmatt,3);
                        pmimp07(storenum,:) = impm(:,1)';
                        gmimp07(storenum,:) = impm(:,2)';
                        rmimp07(storenum,:) = impm(:,3)';
                        
                        vdecs = vardecomp(BTmat(:,t),Cmatt,Dmatt,1);
                        psvdec07(storenum,:) = vdecs(:,1)';
                        gsvdec07(storenum,:) = vdecs(:,2)';
                        rsvdec07(storenum,:) = vdecs(:,3)';
                        
                        vdecd = vardecomp(BTmat(:,t),Cmatt,Dmatt,2);
                        pdvdec07(storenum,:) = vdecd(:,1)';
                        gdvdec07(storenum,:) = vdecd(:,2)';
                        rdvdec07(storenum,:) = vdecd(:,3)';
                        
                        vdecm = vardecomp(BTmat(:,t),Cmatt,Dmatt,3);
                        pmvdec07(storenum,:) = vdecm(:,1)';
                        gmvdec07(storenum,:) = vdecm(:,2)';
                        rmvdec07(storenum,:) = vdecm(:,3)';
              
                    end
                    
                end
        
            end
            
            if discard == 1
            
                BTback(:,:,2:MaxBacksteps) = BTback(:,:,1:MaxBacksteps-1);
                BTback(:,:,1) = BTmat;
        
                ATback(:,:,2:MaxBacksteps) = ATback(:,:,1:MaxBacksteps-1);
                ATback(:,:,1) = ATmat;
                
                lSigTback(:,:,2:MaxBacksteps) = lSigTback(:,:,1:MaxBacksteps-1);
                lSigTback(:,:,1) = lSigTmat;
                
                Qback(:,:,2:MaxBacksteps) = Qback(:,:,1:MaxBacksteps-1);
                Qback(:,:,1) = Q;
                
                Sback(:,:,2:MaxBacksteps) = Sback(:,:,1:MaxBacksteps-1);
                Sback(:,:,1) = S;
                
                Wback(:,:,2:MaxBacksteps) = Wback(:,:,1:MaxBacksteps-1);
                Wback(:,:,1) = W;     
                
                AvailBacksteps = min([ AvailBacksteps+1 MaxBacksteps ]);
                
            end
            
            sweep = sweep + 1;
            
        end
        
    end
    
end

Baccept_Gibbs = Baccept - Baccept_burnin;
Breject_Gibbs = Breject - Baccept_burnin;

sigaccept_Gibbs = sigaccept - sigaccept_burnin;
sigreject_Gibbs = sigreject - sigreject_burnin;

Signaccept_Gibbs = Signaccept - Signaccept_burnin;
Signreject_Gibbs = Signreject - Signreject_burnin;

% =========================================================================
% Compute convergence diagonstics
% =========================================================================

mbar = 24;
% lag truncation parameter for estimating frequency-zero spectral density

BTCD = zeros(3+9*q,T);
BTIF = zeros(3+9*q,T);

for bind = 1:3+9*q
    
    for tind = 1:T
        
        thetadraws = reshape(betaT(bind,tind,:),nstores,1);
        
        [ ConDia,InefFact ] = convstats(thetadraws,mbar);
        
        BTCD(bind,tind) = ConDia;
        BTIF(bind,tind) = InefFact;
        
    end
    
end

ATCD = zeros(3,T);
ATIF = zeros(3,T);

for aind = 1:3
    
    for tind = 1:T
        
        if aind == 1
            
            thetadraws = alphagpT(:,tind);
            
        elseif aind == 2
            
            thetadraws = alpharpT(:,tind);
            
        else
            
            thetadraws = alphargT(:,tind);
            
        end
        
        [ ConDia,InefFact ] = convstats(thetadraws,mbar);
        
        ATCD(aind,tind) = ConDia;
        ATIF(aind,tind) = InefFact;

    end

end

SIGTCD = zeros(3,T);
SIGTIF = zeros(3,T);

for sind = 1:3
    
    for tind = 1:T
        
        if sind == 1
            
            thetadraws = sigpT(:,tind);
            
        elseif sind == 2
            
            thetadraws = siggT(:,tind);
            
        else
            
            thetadraws = sigrT(:,tind);
            
        end
        
        [ ConDia,InefFact ] = convstats(thetadraws,mbar);
        
        SIGTCD(sind,tind) = ConDia;
        SIGTIF(sind,tind) = InefFact;

    end

end

QTCD = zeros((3+9*q)*(3+9*q+1)/2,1);
QTIF = zeros((3+9*q)*(3+9*q+1)/2,1);

for qind = 1:(3+9*q)*(3+9*q+1)/2
    
    thetadraws = QT(qind,:)';
    
    [ ConDia,InefFact ] = convstats(thetadraws,mbar);
        
    QTCD(qind) = ConDia;
    QTIF(qind) = InefFact;
    
end

thetadraws = S1T';
[ ConDia,InefFact ] = convstats(thetadraws,mbar);
S1TCD = ConDia;
S1TIF = InefFact;

S2TCD = zeros(3,1);
S2TIF = zeros(3,1);

for s2ind = 1:3
    
    thetadraws = S2T(s2ind,:)';
    
    [ ConDia,InefFact ] = convstats(thetadraws,mbar);

    S2TCD(s2ind) = ConDia;
    S2TIF(s2ind) = InefFact;
    
end

WTCD = zeros(3,1);
WTIF = zeros(3,1);

for wind = 1:3
    
    thetadraws = WT(wind,:)';
    
    [ ConDia,InefFact ] = convstats(thetadraws,mbar);

    WTCD(wind) = ConDia;
    WTIF(wind) = InefFact;
    
end

% =========================================================================
% Save results
% =========================================================================

save('betaT.mat','betaT','-v7.3');
clear betaT;
save('tvpvarout.mat');
